package com.gemserk.games.taken;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.gemserk.commons.svg.inkscape.SvgDocument;
import com.gemserk.commons.svg.inkscape.SvgDocumentHandler;
import com.gemserk.commons.svg.inkscape.SvgInkscapeGroup;
import com.gemserk.commons.svg.inkscape.SvgInkscapeGroupHandler;
import com.gemserk.commons.svg.inkscape.SvgInkscapeImage;
import com.gemserk.commons.svg.inkscape.SvgInkscapeImageHandler;
import com.gemserk.commons.svg.inkscape.SvgParser;
import com.gemserk.vecmath.Matrix3f;
import com.gemserk.vecmath.Vector3f;

public class WorldLoader {

	private SvgDocument svgDocument;

	private String layer;
	
	public WorldLoader(String layer) {
		this.layer = layer;
	}

	public void processWorld(Document document) {
		SvgParser svgParser = new SvgParser();
		svgParser.addHandler(new SvgDocumentHandler() {
			@Override
			protected void handle(SvgParser svgParser, SvgDocument svgDocument, Element element) {
				WorldLoader.this.svgDocument = svgDocument;
			}
		});
		svgParser.addHandler(new SvgInkscapeGroupHandler() {

			@Override
			protected void handle(SvgParser svgParser, SvgInkscapeGroup svgInkscapeGroup, Element element) {

				if (isInkscapeLayer(svgInkscapeGroup) && !isLayer(svgInkscapeGroup)) {
					svgParser.processChildren(false);
					return;
				}

			}

			private boolean isLayer(SvgInkscapeGroup svgInkscapeGroup) {
				return svgInkscapeGroup.getLabel().equalsIgnoreCase(layer);
			}

			private boolean isInkscapeLayer(SvgInkscapeGroup svgInkscapeGroup) {
				return svgInkscapeGroup.getGroupMode().equals("layer");
			}

		});
		svgParser.addHandler(new SvgInkscapeImageHandler() {

			private boolean isFlipped(Matrix3f matrix) {
				return matrix.getM00() != matrix.getM11();
			}

			@Override
			protected void handle(SvgParser svgParser, SvgInkscapeImage svgImage, Element element) {

				if (svgImage.getLabel() == null)
					return;

				float width = svgImage.getWidth();
				float height = svgImage.getHeight();

				Matrix3f transform = svgImage.getTransform();

				Vector3f position = new Vector3f(svgImage.getX() + width * 0.5f, svgImage.getY() + height * 0.5f, 0f);
				transform.transform(position);

				Vector3f direction = new Vector3f(1f, 0f, 0f);
				transform.transform(direction);

				float angle = 360f - (float) (Math.atan2(direction.y, direction.x) * 180 / Math.PI);

				float sx = 1f;
				float sy = 1f;

				if (isFlipped(transform)) {
					sy = -1f;
				}

				float x = position.x;
				float y = svgDocument.getHeight() - position.y;
				
				if (element.hasAttribute("spawn")) {
					String spawnType = element.getAttribute("spawn");
					
					if ("deadRobot".equalsIgnoreCase(spawnType)) {
						handleDeadRobot(x, y, angle);
					}
					
				}

				if (element.hasAttribute("start"))
					handleCharacterStartPoint(x, y);
				else if (element.hasAttribute("robotStart"))
					handleRobotStartPoint(x, y);
				else
					handleStaticObject(svgImage, width, height, angle, sx, sy, x, y);

			}

		});
		svgParser.parse(document);
	}

	protected void handleCharacterStartPoint(float x, float y) {
		
	}
	
	protected void handleDeadRobot(float x, float y, float angle) {
		
	}

	protected void handleRobotStartPoint(float x, float y) {
		
	}

	protected void handleStaticObject(SvgInkscapeImage svgImage, float width, float height, float angle, float sx, float sy, float x, float y) {
		
	}

}